!{\src2tex{textfont=tt}}
!!****f* ABINIT/calc_dij0
!! NAME
!! calc_dij0
!!
!! FUNCTION
!! Compute "frozen" values of Dij = Dij0
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  pawarray
!!    %indlmn(6,lmn_size)= Gives l,m,n,lm,ln,s for i=lmn
!!    %shapefunc(wav_meshsz)= Normalized shape function
!!  pawrad=datastructure containing mesh size definitions
!!  pawps
!!    %phi(wav_meshsz,basis_size)= PAW atomic wavefunctions on the radial grid
!!    %tphi(wav_meshsz,basis_size)= PAW atomic pseudo-wavefunctions on the radial grid
!!    %vhtnzc(max_meshsz)= Hartree potential of the ps-density
!!                         of the nucleus + core electrons
!!  pshead
!!    %log_step= Logarithmic step corresponding to radial mesh
!!    %lmn_size= Number of elements for the paw basis
!!    %lmn2_size= lmn_size*(lmn_size+1)/2
!!    %sph_meshsz= Dimension of radial mesh for spheres
!!    %wav_meshsz=Mesh size for partial waves and projectors inherited from uspp
!!  usdata
!!    %eig(basis_size)= eigen- or refernce energy for each partial wave
!!    %rvae(max_meshsz)= AE atomic potential from uspp
!!
!! OUTPUT
!!  pawps
!!    %dij0(lmn2_size)= Frozen part of the Dij term
!!    %kij(lmn2_size)= Kinetic overlap operator in partial waves basis
!!
!! PARENTS
!!      uspp2abinit
!!
!! CHILDREN
!!      csimp,deducer0,nderiv,poisson
!!
!! SOURCE

 subroutine calc_dij0(pawarray,pawrad,pawps,pshead,usdata)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ---------------------------------------------
 type(pawarray_type) :: pawarray
 type(pawrad_type)   :: pawrad
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead
 type(usdata_type)   :: usdata

!Local variables ---------------------------------------
 integer :: ilm,iln,ilmn,j0lmn,jlm,jln,jlmn,klmn,ll,meshsz,meshszm,meshszw
 real(dp) :: llp2,intg,intvh
 real(dp),allocatable :: ff(:),dtphj(:),d2tphj(:),vhnzc(:)

! *************************************************************************

 meshsz =pshead%sph_meshsz
 meshszm=pshead%max_meshsz
 meshszw=pshead%wav_meshsz
 allocate(ff(meshsz))

!Kinetic part of Dij0
!====================
 pawarray%kij=zero
 allocate(dtphj(meshszw),d2tphj(meshszw))
 do jlmn=1,pshead%lmn_size
  j0lmn=jlmn*(jlmn-1)/2
  jlm=pawarray%indlmn(4,jlmn);jln=pawarray%indlmn(5,jlmn)
  ll=pawarray%indlmn(1,jlmn);llp2=dble(ll*(ll+1))
  call nderiv(meshszw,pawrad%lstep,pawrad%radfact(1:meshszw),pawps%tphi(1:meshszw,jln),dtphj)
  call nderiv(meshszw,pawrad%lstep,pawrad%radfact(1:meshszw),dtphj,d2tphj)
  do ilmn=1,jlmn
   klmn=j0lmn+ilmn
   ilm=pawarray%indlmn(4,ilmn);iln=pawarray%indlmn(5,ilmn)
   if (jlm==ilm) then
    ff(2:meshsz)=pawps%phi(2:meshsz,iln)*pawps%phi(2:meshsz,jln) &
&              *(usdata%eig(jln)-two*usdata%rvae(2:meshsz)/pawrad%rad(2:meshsz)) &
&               +pawps%tphi(2:meshsz,iln) &
&                *(d2tphj(2:meshsz)-llp2*pawps%tphi(2:meshsz,jln)/pawrad%rad(2:meshsz)**2)
    call deducer0(ff,pawrad)
    call csimp(ff,pawrad,meshsz,intg)
    pawarray%kij(klmn)=half*intg
   endif
  enddo
 enddo
 deallocate(dtphj,d2tphj)
 pawps%dij0(1:pshead%lmn2_size)=pawarray%kij(1:pshead%lmn2_size)

!Computation of <phi_i|vh(nZc)|phi_j>-<tphi_i|vh(tnZc)|tphi_j>
!=============================================================
 allocate(vhnzc(meshszm))
 call poisson(pawarray%coreden4pr2,meshszm,pawrad,vhnzc)
 vhnzc(2:meshszm)=(vhnzc(2:meshszm)-pshead%atomic_charge)/pawrad%rad(2:meshszm)
 call deducer0(vhnzc,pawrad)
 do jlmn=1,pshead%lmn_size
  j0lmn=jlmn*(jlmn-1)/2
  jlm=pawarray%indlmn(4,jlmn);jln=pawarray%indlmn(5,jlmn)
  do ilmn=1,jlmn
   klmn=j0lmn+ilmn
   ilm=pawarray%indlmn(4,ilmn);iln=pawarray%indlmn(5,ilmn)
   if (jlm==ilm) then
    ff(1:meshsz)=pawps%phi (1:meshsz,iln)*pawps%phi (1:meshsz,jln)*vhnzc(1:meshsz) &
&               -pawps%tphi(1:meshsz,iln)*pawps%tphi(1:meshsz,jln)*pawps%vhtnzc(1:meshsz)
    call csimp(ff,pawrad,meshsz,intg)
    pawps%dij0(klmn)=pawps%dij0(klmn)+intg
   endif
  enddo
 enddo
 deallocate(vhnzc)

!Computation of -int[vh(tnzc)*Qijhat(r)dr]
!=========================================
 ff(1:meshsz)=pawps%vhtnzc(1:meshsz)*pawarray%shapefunc(1:meshsz)*pawrad%rad(1:meshsz)**2
 call csimp(ff,pawrad,meshsz,intvh)
 do jlmn=1,pshead%lmn_size
  j0lmn=jlmn*(jlmn-1)/2
  jln=pawarray%indlmn(5,jlmn);jlm=pawarray%indlmn(4,jlmn)
  do ilmn=1,jlmn
   klmn=j0lmn+ilmn
   iln=pawarray%indlmn(5,ilmn);ilm=pawarray%indlmn(4,ilmn)
   if (ilm==jlm) then
    ff(1:meshsz)=(pawps%phi (1:meshsz,iln)*pawps%phi (1:meshsz,jln)&
&                -pawps%tphi(1:meshsz,iln)*pawps%tphi(1:meshsz,jln))
    call csimp(ff,pawrad,meshsz,intg)
    pawps%dij0(klmn)=pawps%dij0(klmn)-intvh*intg
   endif
  enddo
 enddo

 deallocate(ff)

 end subroutine
!!***
