!{\src2tex{textfont=tt}}
!!****f* ABINIT/calc_valden
!! NAME 
!! calc_valden
!!
!! FUNCTION
!! Compute pseudo valence density (ntild) and compensation density (nhat) from USPP
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the 
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  pawrad=datastructure containing mesh size definitions
!!  pshead
!!    %basis_size= Number of elements for the paw nl basis
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %max_meshsz= Maximal mesh size for densities/potentials
!!    %wav_meshsz= Mesh size for partial waves and projectors inherited from uspp
!!  usdata
!!    %lval(nval)= l angular momentum for each valence state
!!    %nval= Number of valence states
!!    %occ(nval)= Valence states occupancies
!!    %qijps(basis_size,nval)= Pseudized qij quantities(phi.phj-tphi.tphj)
!!    %rsvale(max_meshsz)= (Pseudo valence density + hat density) multiplied by 4Pi.r2 (from USPP)
!!    %snlprj(basis_size,nval)= Projected eigentstates, i.e. <Snl(j)|Proj(i)> scalars
!!  un_log= Unit number for log file (comments)
!!
!! OUTPUT
!!  pawarray
!!    %shpnrm(0)= Norm of compensation density
!!    %tvaleden4pr2(max_meshsz)= Pseudo valence density multiplied by 4Pi.r2  (read from uspp)
!!    %ushatden4pr2(wav_meshsz)= Compensation density (from USPP) multiplied by 4Pi.r2
!!  pawps
!!    %tvaledens(pshead%vale_meshsz)= Pseudo-valence density of the atom
!!
!! PARENTS
!!      uspp2abinit
!!
!! CHILDREN
!!      csimp
!!
!! SOURCE

 subroutine calc_valden(pawarray,pawrad,pawps,pshead,usdata,un_log)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ------------------------------------
 integer :: un_log
 type(pawarray_type) :: pawarray
 type(pawrad_type)   :: pawrad
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead
 type(usdata_type)   :: usdata

!Local variables-------------------------------
 real(dp),parameter :: vale_prec=1.d-10
 integer :: ib,ir,isize,ival,jb,meshsz
 logical :: found

! *************************************************************************

 meshsz=pshead%wav_meshsz

! Compute hat density from USPP
  pawarray%ushatden4pr2(:)=zero
  do ival=1,usdata%nval
   do jb=1,pshead%basis_size
    if (usdata%lval(ival)==pshead%orbitals(jb)) then
     do ib=1,pshead%basis_size
      if (usdata%lval(ival)==pshead%orbitals(ib)) then
       pawarray%ushatden4pr2(1:meshsz)=pawarray%ushatden4pr2(1:meshsz) &
&                          +usdata%qijps(1:meshsz,ib,jb) &
&                          *usdata%snlprj(ib,ival)*usdata%snlprj(jb,ival) &
&                          *usdata%occ(ival)
      endif
     enddo
    endif
   enddo
  enddo

!Compute norm of compensation density
 call csimp(pawarray%ushatden4pr2,pawrad,meshsz,pawarray%shpnrm(0))

!Substract hatden from rsvale
 pawarray%tvaleden4pr2(1:pshead%max_meshsz)=usdata%rsvale(1:pshead%max_meshsz)
 pawarray%tvaleden4pr2(1:meshsz)=pawarray%tvaleden4pr2(1:meshsz)-pawarray%ushatden4pr2(1:meshsz)

!Evaluate tvaledens for r>0
 do ir=2,pshead%vale_meshsz
  pawps%tvaledens(ir)= pawarray%tvaleden4pr2(ir)/(four_pi*pawrad%rad(ir)**2)
 enddo
 call deducer0(pawps%tvaledens,pawrad)

!Adjust mesh size for (t)vale density in order to get tvaldens lower than given precision
 isize=pshead%vale_meshsz
 do while (abs(pawps%tvaledens(isize))<vale_prec.and.isize>1)
  isize=isize-1
 enddo
 pshead%vale_meshsz=max(min(isize+1,pshead%vale_meshsz),pshead%sph_meshsz)
 if (pshead%vale_meshsz>pshead%sph_meshsz) then
  write(un_log,'(/,a)') '> USpp->Abinit translator INFO:'
  write(un_log,'(a,i4)') '  Mesh size for valence density has been adjusted to ',pshead%vale_meshsz
 endif

end subroutine
!!***
