!{\src2tex{textfont=tt}}
!!****f* ABINIT/calc_vxc
!! NAME
!! calc_vxc
!!
!! FUNCTION
!! Compute exchange-correlation potential*r from input density
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (FJ,MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  den4pr2(meshsz)= input density*4*pi*r^2
!!  exfact= USpp s code number for the exchange-correlation
!!  meshsz = dimension of the radial mesh (from 0 to infinity)
!!  pawrad=datastructure containing mesh size definitions
!!  un_log= Unit number for log file (comments)
!!
!! OUTPUT
!!  rvxc(meshsz)= r*exchange-correlation potential
!!
!! SIDE EFFECTS
!!
!!
!! NOTES
!!  This routine is inspired from Vanderbilt ultrasoft code (uspp)
!!
!! PARENTS
!!      calc_vhtnzc
!!
!! CHILDREN
!!      csimp,deducer0,deriv
!!
!! SOURCE

 subroutine calc_vxc(den4pr2,meshsz,pawrad,rvxc,exfact,un_log)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ---------------------------------------------
 integer :: exfact,meshsz,un_log
 real(dp) :: den4pr2(meshsz),rvxc(meshsz)

!These types are defined in defs_pawps
 type(pawrad_type) :: pawrad

!Local variables ---------------------------------------
 real(dp), parameter :: rhoc1=1.d-10,rhoc2=1.d-20
 integer :: iexp1,iexp2,ir,nneg
 real(dp) :: aa,bb,cc,coef1,coef2,excc,uxcc,intn1,intnn
 real(dp), allocatable :: nn(:),dndr(:),dndr2(:)

!-------------------------------------------------------

!Take measures to ensure that den is greater than zero
 nneg=count(den4pr2<-1.d-30)
 if (nneg/=0) then
  write(un_log,'(/,a)') '> USpp->Abinit translator ERROR (calc_vxc):'
  write(un_log,'(a,i4,a)')   '  Negative density at',nneg,' r-values !'
  stop 'Program stopped before end'
 endif

 allocate(nn(meshsz),dndr(meshsz),dndr2(meshsz))

!Compute total charge density ->fu()
 nn(2:meshsz)=den4pr2(2:meshsz)/(four_pi*pawrad%rad(2:meshsz)**2)
 call deducer0(nn,pawrad)

!Compute the 1. derivative ->xi
 call nderiv(meshsz,pawrad%lstep,pawrad%radfact,nn,dndr)
!Compute the 2. derivative ->xj
 call nderiv(meshsz,pawrad%lstep,pawrad%radfact,dndr,dndr2)

!Specify cutoff density below which the derivatives are zero
 iexp1=1;iexp2=1
 do ir=2,meshsz
  if (nn(ir)>rhoc1) iexp1=ir
  if (nn(ir)>rhoc2) iexp2=ir
 enddo

!Fit exponential tail to the density
 if(iexp1<(meshsz-2)) then
  call csimp(nn,pawrad,iexp1,intn1)
  call csimp(nn,pawrad,meshsz,intnn)
  bb=(dndr(iexp1)/nn(iexp1)+dndr(iexp1-1)/nn(iexp1-1)+dndr(iexp1+1)/nn(iexp1+1))/3.d0
  cc=-bb*pawrad%rad(iexp1)
  aa=-(intnn-intn1)/((cc*cc+2.d0*cc+2.d0)*exp(-cc)*four_pi)*bb**3
  do ir=iexp1,iexp2
   nn(ir)=aa*exp(bb*pawrad%rad(ir))
   dndr(ir)=bb*nn(ir)
   dndr2(ir)=bb*dndr(ir)
  enddo
 endif

!Put the density to zero beyond r(iexp2)
 do ir=iexp2,meshsz;dndr(ir)=zero;dndr2(ir)=zero;enddo
 call xctype(exfact,rhoc2,0.d0,0.d0,pawrad%rad(iexp2),uxcc,excc)
 coef1=( 3.d0*excc-uxcc)/rhoc2
 coef2=(-2.d0*excc+uxcc)/rhoc2**2

!Compute exchange-correlation related to total charge density
 rvxc(1)=zero
 do ir=2,meshsz
  if (nn(ir)>rhoc2) then
   call xctype(exfact,nn(ir),dndr(ir),dndr2(ir),pawrad%rad(ir),uxcc,excc)
  else
   uxcc=2.d0*coef1*nn(ir)+3.d0*coef2*nn(ir)**2
  endif
  rvxc(ir)=pawrad%rad(ir)*uxcc*0.5d0
 enddo

 deallocate(nn,dndr,dndr2)

 end subroutine
!!***
