!{\src2tex{textfont=tt}}
!!****f* ABINIT/dotest
!! NAME
!! dotest
!!
!! FUNCTION
!! This routine perform some tests on quantities inherited from USPP
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  pawarray
!!    %tvaleden4pr2(max_meshsz)= Pseudo valence density multiplied by 4Pi.r2
!!    %tcoreden4pr2(max_meshsz)= Pseudo core density multiplied by 4Pi.r2
!!  pawrad=datastructure containing mesh size definitions
!!  pshead
!!    %basis_size= Number of elements for the paw nl basis
!!    %log_step= Logarithmic step corresponding to radial mesh
!!    %max_meshsz= Maximal mesh size for densities/potentials
!!    %rad_step= Radial step corresponding to radial mesh
!!    %wav_meshsz=Mesh size for partial waves and projectors inherited from uspp
!!  un_log= Unit number for log file (comments)
!!  usdata
!!    %rsvale(max_meshsz)= (Pseudo valence density + hat density) multiplied by 4Pi.r2 (from USPP)
!!
!! SIDE EFFECTS
!!  pawps
!!    %phi(wav_meshsz,basis_size)= atomic partial waves
!!    %tphi(wav_meshsz,basis_size)= atomic pseudo partial waves
!!    %tproj(wav_meshsz,basis_size)= projectors on partial waves
!!  pshead
!!    %rc_sph= Default PAW sphere radius
!!    %sph_meshsz=Dimension of radial mesh for projectors
!!    %sph_meshsz=Dimension of radial mesh for spheres
!!
!! PARENTS
!!      uspp2abinit
!!
!! CHILDREN
!!      csimp
!!
!! SOURCE

 subroutine dotest(pawarray,pawps,pawrad,pshead,un_log,usdata)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ------------------------------------
 integer :: un_log
 type(pawarray_type) :: pawarray
 type(pawps_type)    :: pawps
 type(pawrad_type)   :: pawrad
 type(pshead_type)   :: pshead
 type(usdata_type)   :: usdata

!Local variables-------------------------------
 integer :: ib,ierrs,ierrp,s_meshsz,p_meshsz,negc,negv
 real(dp) :: intg1,intg2,xx,yy
 real(dp),allocatable :: den(:)


! *************************************************************************

 allocate(den(pshead%max_meshsz))

!Test equality of pseudized and original quantities at r=Rpaw
 s_meshsz=pshead%sph_meshsz;ierrs=0
 p_meshsz=pshead%prj_meshsz;ierrp=0
 do ib=1,pshead%basis_size
  xx=abs(pawps%phi(s_meshsz,ib)-pawps%tphi(s_meshsz,ib))
  if (xx>1.d-8) then
   if (s_meshsz==pshead%sph_meshsz) then
    if (s_meshsz<pshead%wav_meshsz) then
     s_meshsz=s_meshsz+1
     xx=abs(pawps%phi(s_meshsz,ib)-pawps%tphi(s_meshsz,ib))
     if (xx>1.d-15) ierrs=1
    else
     ierrs=1
    endif
   else
    ierrs=1
   endif
  endif
  yy=abs(pawps%tproj(p_meshsz,ib))
  do while ((yy>1.d-5).and.(p_meshsz<pshead%wav_meshsz))
   p_meshsz=p_meshsz+1
   yy=abs(pawps%tproj(p_meshsz,ib))
  end do
  if (yy>1.d-15) ierrp=1
 enddo
 if (ierrs>0) then
  write(un_log,'(/,a)') '> USpp->Abinit translator ERROR (dotest):'
  write(un_log,'(a)')   '    Phi and tPhi localized orbitals are'
  write(un_log,'(a)')   '    not equal for r>R_paw !'
  stop 'Program stopped before end'
 else if (ierrp>0) then
  write(un_log,'(/,a)') '> USpp->Abinit translator ERROR (dotest):'
  write(un_log,'(a)')   '    Non-local projectors are not zero for r>R_paw !'
  stop 'Program stopped before end'
 else 
  pshead%sph_meshsz=s_meshsz
  pshead%prj_meshsz=p_meshsz
  pshead%rc_sph=pshead%rad_step*(exp(pshead%log_step*dble(s_meshsz-1))-1.d0)
 endif
 do ib=1,pshead%basis_size
  pawps%tphi(pshead%sph_meshsz,ib)=pawps%phi(pshead%sph_meshsz,ib)
  pawps%tproj(pshead%prj_meshsz,ib)=zero
 enddo

!Test negative densities
 den(1:pshead%max_meshsz)=pawarray%tcoreden4pr2(1:pshead%max_meshsz)+usdata%rsvale(1:pshead%max_meshsz)
 negc=count(pawarray%tcoreden4pr2<-1.d-30);negv=count(den<-1.d-30)
 if (negv>5) then
  write(un_log,'(/,a)') '> USpp->Abinit translator ERROR:'
  write(un_log,'(a,i3,a)')   '    Pseudized (core+valence) density went negative at ',negv,' points !'
  write(un_log,'(a)')   '    Verify input parameters (rpcor, rpinner, ..)...'
  stop 'Program stopped before end'
 else if (negc>5) then
  write(un_log,'(/,a)') '> USpp->Abinit translator WARNING:'
  write(un_log,'(a,i3,a)')   '  Pseudized core density went negative at ',negc,' points !'
  write(un_log,'(a)')   '  Verify input parameters (in particular rinner and rpcor) !!!!!'
  write(un_log,'(a)')   'Assume experienced user; program do not stop...'
 endif

!Test compensation charge
 den(1:pshead%max_meshsz)=pawarray%valeden4pr2(1:pshead%max_meshsz)-usdata%rsvale(1:pshead%max_meshsz)
 call csimp(den,pawrad,pshead%sph_meshsz,intg1)
 call csimp(den,pawrad,pshead%max_meshsz,intg2)
 write(un_log,'(/,a)') '> USpp->Abinit translator INFO:'
 write(un_log,'(a,2(g8.1,a))')   '  Intg[n-tn-nhat]=',intg1,' (inside sph.), ',intg2,' (whole space)'

 deallocate(den)

end subroutine
!!***
