!{\src2tex{textfont=tt}}
!!****f* ABINIT/initpawps
!! NAME
!! initpawps
!!
!! FUNCTION
!! Initialize several quantities
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  pshead
!!    %basis_size= Number of elements for the paw nl basis
!!    %lmn_size= Number of elements for the paw basis
!!    %log_step= Logarithmic step corresponding to radial mesh
!!    %max_meshsz=Maximal mesh size for densities/potentials
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %prj_meshsz=Dimension of radial mesh for spheres inherited from uspp
!!    %pspxc_vdb= Vdb s code number for the exchange-correlation
!!    %rad_step= Radial step corresponding to radial mesh
!!    %rc_sph= Default PAW sphere radius
!!    %sph_meshsz=Dimension of radial mesh for spheres
!!    %wav_meshsz=Mesh size for partial waves and projectors inherited from uspp
!!  pawopt
!!   %ishapenum=Option governing the writing of shape functions
!!   %ishapetyp=Option governing the type of the shape function k(r)
!!   %rcompfact=Radius for compensation charge (-1 if rcomp=rcut)
!!   %usexcnhat=Option governing the use of nhat in XC terms
!!  un_log= Unit number for log file (comments)
!!  usdata
!!    %exfact= USpp s code number for the exchange-correlation
!!    %rloc= Cut-off radius for Vloc pseudization
!!  version= version number for USPP2Abinit
!!
!! OUTPUT
!!  pawarray
!!    %indlmn(6,lmn_size)= Gives l,m,n,lm,ln,s for i=lmn
!!  pawrad
!!    %max_meshsz= Max. dimension of radial mesh
!!  pshead
!!    %core_meshsz= Dimension of radial mesh for core and tcore densities
!!    %creatorid= ID of psp generator (here pspfmt=1 !)
!!    %l_size= Max. value of l+1 leading to a non zero Gaunt coeffs
!!    %lmax= Maximum value of l
!!    %lmn2_size= lmn_size*(lmn_size+1)/2
!!    %mesh_type=  Flag defining ther radial grid type
!!    %prj_msz_max= Max. dimension of radial mesh for tproj
!!    %pspcod= Psp code number for Abinit (here PAW->pspcod=7 !)
!!    %pspxc_abinit= Abinit s code number for the exchange-correlation
!!    %rc_sph= Default PAW sphere radius
!!    %shape_type= Shape function type (necessarily 2 or -1 here)
!!    %vale_meshsz= Dimension of radial mesh for vale and tvale densities
!!    %vbar_meshsz= Dimension of radial mesh for Vbare
!!    %vloc_meshsz= Dimension of radial mesh for vloc=vhtnzc
!!    %vlocopt= option for the formulation of Vloc
!!
!! PARENTS
!!      uspp2abinit
!!
!! SOURCE

 subroutine initpawps(pawarray,pawopt,pawrad,pshead,un_log,usdata,version)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ---------------------------------------------
 integer :: un_log
 character*5 ::version
 type(pawarray_type) :: pawarray
 type(pawopt_type)   :: pawopt
 type(pawrad_type)   :: pawrad
 type(pshead_type)   :: pshead
 type(usdata_type)   :: usdata

!Local variables ---------------------------------------
 real(dp),parameter :: rmax_vloc=10.d0 !We decide to cut at r=10 u.a because of numeric noise...

 integer :: ii1,ii2,ii3,ib,il,ilm,ilmn,iln,nproj
 real(dp) :: rctest

!--------------------------------------------------------

!Format codes for Abinit pseudopotential
 read(unit=version(1:1),fmt='(i1)') ii1
 read(unit=version(3:3),fmt='(i1)') ii2
 read(unit=version(5:5),fmt='(i1)') ii3
 pshead%creatorid=2000+100*ii1+10*ii2+ii3
 pshead%pspcod=7

!Grid type (here, logarithmic grid)
 pshead%mesh_type=2

!Maximum value for l quantum number
 pshead%lmax=maxval(pshead%orbitals(1:pshead%basis_size))
 pshead%l_size=2*pshead%lmax+1
 pshead%lmn2_size=pshead%lmn_size*(pshead%lmn_size+1)/2

!Radius for PAW spheres
 rctest=pshead%rad_step*(exp(pshead%log_step*dble(pshead%sph_meshsz-1))-1.d0)
 if (abs(rctest-pshead%rc_sph)>tol6) then
  write(un_log,'(/,a)') '> USpp->Abinit translator ERROR (initpawps):'
  write(un_log,'(a)')   '    Inconsistency between sphere radius'
  write(un_log,'(a)')   '    and mesh size !'
  write(un_log,'(2(a,f10.6))')'  r(meshsz)=',rctest,', rc=',pshead%rc_sph
  stop 'Program stopped before end'
 endif
 pshead%rc_sph=rctest

!Sizes of radial meshes
 pshead%prj_msz_max=max(pshead%prj_meshsz*3,pshead%wav_meshsz)
 pshead%core_meshsz=pshead%max_meshsz
 pshead%vale_meshsz=pshead%max_meshsz
 pshead%vloc_meshsz=max(pshead%wav_meshsz,pshead%vbar_meshsz,min(pshead%max_meshsz, &
&                       int(log(1.d0+rmax_vloc/pshead%rad_step)/pshead%log_step)+1))
 pawrad%max_meshsz=max(pshead%prj_msz_max,pshead%max_meshsz,pshead%wav_meshsz, &
&                      pshead%core_meshsz,pshead%vale_meshsz,pshead%vloc_meshsz)

!Initialization of the orbital basis indexes (indlmn)
 ilmn=0;iln=0
 do il=0,pshead%lmax
  nproj=0
  do ib=1,pshead%basis_size
   if (pshead%orbitals(ib)==il) then
    nproj=nproj+1;iln=iln+1
    do ilm=1,2*il+1
     pawarray%indlmn(1,ilmn+ilm)=il
     pawarray%indlmn(2,ilmn+ilm)=ilm-(il+1)
     pawarray%indlmn(3,ilmn+ilm)=nproj
     pawarray%indlmn(4,ilmn+ilm)=il*il+ilm
     pawarray%indlmn(5,ilmn+ilm)=iln
     pawarray%indlmn(6,ilmn+ilm)=1
    enddo
    ilmn=ilmn+2*il+1
   endif
  enddo
 enddo

!Translation of the exchange-correlation type
 if (usdata%exfact==0.) then
  pshead%pspxc_abinit=2   ! LDA with CA functional
 elseif (usdata%exfact==-1.) then
  pshead%pspxc_abinit=4   ! LDA with Wigner functional
 elseif (usdata%exfact==-2.) then
  pshead%pspxc_abinit=5   ! LDA with Hedin-Lundquist functional
 elseif (usdata%exfact==5.) then
  pshead%pspxc_abinit=11  ! GGA with PBE functional
 else
  write(un_log,'(/,a)') '> USpp->Abinit translator ERROR (initpawps):'
  write(un_log,'(a)')   '    Incorrect value of ex-correlation code'
  write(un_log,'(a)')   '    (not supported by Abinit) !'
  stop 'Program stopped before end'
 endif

!Shape function type
 if (pawopt%ishapetyp>0) then
  pshead%shape_type=1+pawopt%ishapetyp
  if (pawopt%ishapenum==1) pshead%shape_type=-1
 else
  pshead%shape_type=-1
 endif

!Option for Vloc
 pshead%vlocopt=1
 if (pawopt%usexcnhat==0) pshead%vlocopt=2

 end subroutine
!!***
