!{\src2tex{textfont=tt}}
!!****f* ABINIT/rdusps2
!! NAME
!! rdusps2
!!
!! FUNCTION
!! Read various usefull data (arrays) from USpp code (D. Vanderbilt)
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  pshead
!!    %basis_size= Number of elements for the paw nl basis
!!    %max_meshsz= Maximal mesh size for densities/potentials
!!    %shape_type= Shape function type
!!    %wav_meshsz=Mesh size for partial waves and projectors inherited from uspp
!!  usdata
!!    %nval= Number of valence states
!!
!! OUTPUT
!!  pawarray
!!    %coreden4pr2(max_meshsz)= Core density multiplied by 4Pi.r2
!!    %tcoreden4pr2(max_meshsz)= Pseudo core density multiplied by 4Pi.r2
!!    %valeden4pr2(max_meshsz)= Valence density multiplied by 4Pi.r2
!!  pawps
!!    %phi(wav_meshsz,basis_size)= atomic partial waves
!!    %tphi(wav_meshsz,basis_size)= atomic pseudo partial waves
!!    %tproj(wav_meshsz,basis_size)= projectors on partial waves
!!  un_log= Unit number for log file (comments)
!!  usdata
!!    %aesnl(wav_meshsz,nval)= Eigenfunctions of AE Hamiltonian
!!    %dion(basis_size,basis_size)= descreened atomic Dij quantities
!!    %eig(basis_size)= eigen- or refernce energy for each partial wave
!!    %keyee(basis_size)= indice of reference (eigen)energy for each basis function
!!    %lval(nval)= l angular momentum for each valence state
!!    %occ(nval)= Valence states occupancies
!!    %pawsnl(wav_meshsz,nval)= Eigenfunctions of PAW/US Hamiltonian
!!    %qijps(basis_size,basis_size,nval)= Pseudized qij quantities(phi.phj-tphi.tphj)
!!    %rsvale(max_meshsz)= (Pseudo valence density + hat density) multiplied by 4Pi.r2 (from USPP)
!!    %rvae(max_meshsz)= AE atomic potential from uspp
!!    %rvloc(max_meshsz)= Pseudized potential * r
!!    %rvloc0(max_meshsz)= Descreened pseudized potential * r (from USPP)
!!    %snlprj(basis_size,nval)= Projected eigentstates, i.e. <PAWSnl(j)|Proj(i)> scalars
!!
!! PARENTS
!!      uspp2abinit
!!
!! CHILDREN
!!
!! SOURCE

 subroutine rdusps2(pawarray,pawps,pshead,un_log,usdata)

 use defs_basis
 use defs_pawps

 implicit none

 include "usppcom.inc"

!Arguments ---------------------------------------------
 integer :: un_log
 type(pawarray_type) :: pawarray
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead
 type(usdata_type)   :: usdata

!Local variables ---------------------------------------
 integer :: ib,jb

! *********************************************************************

!EIGENSTATES
 usdata%occ(1:usdata%nval)=usocc(1:usnval)
 usdata%lval(1:usdata%nval)=uslval(1:usnval)
 do ib=1,usnval
  usdata%aesnl(1:pshead%max_meshsz,ib)=usaesnl(1:pshead%max_meshsz,ib)
  usdata%pawsnl(1:pshead%max_meshsz,ib)=ussnl(1:pshead%max_meshsz,ib)
  usdata%snlprj(1:pshead%basis_size,ib)=ussnlprj(1:pshead%basis_size,ib)
 enddo

!WAVE FUNCTIONS AND PROJECTORS
 usdata%eig(1:pshead%basis_size)=useig(1:pshead%basis_size)
 usdata%keyee(1:pshead%basis_size)=uskeyee(1:pshead%basis_size)
 do ib=1,pshead%basis_size
  pawps%phi(1:pshead%wav_meshsz,ib)=usphi(1:usmeshwav,ib)
  pawps%tphi(1:pshead%wav_meshsz,ib)=ustphi(1:usmeshwav,ib)
  pawps%tproj(1:pshead%wav_meshsz,ib)=ustproj(1:usmeshwav,ib)
  do jb=1,pshead%basis_size
   usdata%dion(jb,ib)=usddd0(jb,ib)*0.5d0
   usdata%qijps(1:pshead%wav_meshsz,jb,ib)=usqij(1:usmeshwav,jb,ib)
  enddo
 enddo

!DENSITIES AND POTENTIALS
 pawarray%coreden4pr2(1:pshead%max_meshsz)=uscore(1:usmeshmax)
 pawarray%tcoreden4pr2(1:pshead%max_meshsz)=ustcore(1:usmeshmax)
 pawarray%valeden4pr2(1:pshead%max_meshsz)=usvale(1:usmeshmax)
 usdata%rsvale(1:pshead%max_meshsz)=ustvale(1:usmeshmax)
 usdata%rvae(1:pshead%max_meshsz)=usvae(1:usmeshmax)*0.5d0
 usdata%rvloc(1:pshead%max_meshsz)=usvloc(1:usmeshmax)*0.5d0
 usdata%rvloc0(1:pshead%max_meshsz)=usvloc0(1:usmeshmax)*0.5d0

 end subroutine
!!***
