!{\src2tex{textfont=tt}}
!!****f* ABINIT/uspp2abi
!! NAME
!! uspp2abi
!!
!! FUNCTION
!! Main routine for building an atomic data file for PAW calculations
!! in Abinit, starting from the D. Vanderbilt ultra-soft
!! pseudopotential generator
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! NOTES
!! The present main routine drives the following operations :
!!
!! 1) Read useful data from D. Vanderbilt USPP code
!! 2) Calculate quantities that do not depend from self-consistent PAW
!!    iterations
!! 3) Write an output file to be read as an input PAW atomic data
!!    file in abinit
!!
!!       Units: Energies=Hartree, Lengths=Bohr
!!
!! PARENTS
!!
!! CHILDREN
!!    calc_dij0,calc_kij,calc_rhoij0,calc_shapef,calc_core,calc_shapef,calc_descr,calc_valden,
!!    calc_vhtnzc,dotest,initmesh,initpawps,opt_proj,readdat,rdusps1,rdusps2,wrpawps
!!
!! SOURCE

 subroutine uspp2abi(abflname)

 use defs_basis
 use defs_pawps

 implicit none

!==================================================================
! DECLARATIONS

!Global variables -------------------------------------------------

 character*10 :: version='2.4.0', verdate='may 2010', abinitver='5.4.3+'

 integer, parameter :: abunit=22
 integer            :: un_log

 character*(fnlen) :: empty='',abflname,&
&                     datfile='uspp2abinit.dat'

 type(pawarray_type) :: pawarray
 type(pawopt_type)   :: pawopt
 type(pawps_type)    :: pawps
 type(pawrad_type)   :: pawrad
 type(pawrso_type)   :: pawrso
 type(pshead_type)   :: pshead
 type(usdata_type)   :: usdata

!******************************************************************
!BEGIN EXECUTABLE SECTION

!------------------------------------------------------------------
!---- Read pseudopotential file produced by USPP
!------------------------------------------------------------------
!---- Read sizes of arrays from USPP
 allocate(pshead%orbitals(26))
 call rdusps1(pshead,usdata,un_log,version)

!---- Write info
 write(un_log,'(/,a)')'========================================================'
 write(un_log,'(3a)')  '==   uspp2abinit - v',trim(version),':                            =='
 write(un_log,'(a)')  '==   Use D. Vanderbilt ultrasoft psp generator (uspp) =='
 write(un_log,'(a)')  '==   to produce a PAW atomic data file readable       =='
 write(un_log,'(3a)')  '==   by Abinit (v',trim(abinitver),')                              =='
 write(un_log,'(a)')  '========================================================'

!---- Eventually read additional data file
 call readdat(datfile,pawopt,pawrso,un_log)

!---- Initialize some useful data
 allocate(pawarray%indlmn(6,pshead%lmn_size))
 call initpawps(pawarray,pawopt,pawrad,pshead,un_log,usdata,version)

!---- Allocate memory
 allocate(pawps%phi(pshead%wav_meshsz,pshead%basis_size))
 allocate(pawps%tphi(pshead%wav_meshsz,pshead%basis_size))
 allocate(pawps%tproj(pshead%prj_msz_max,pshead%basis_size))
 allocate(pawps%vhtnzc(pshead%max_meshsz))
 allocate(pawps%dij0(pshead%lmn2_size))
 allocate(pawps%rhoij0(pshead%lmn2_size))
 allocate(pawps%coredens(pshead%max_meshsz))
 allocate(pawps%tcoredens(pshead%max_meshsz))
 allocate(pawps%tvaledens(pshead%max_meshsz))
 allocate(pawarray%factscale(pshead%basis_size))
 allocate(pawarray%shapefunc(pshead%wav_meshsz))
 allocate(pawarray%shpnrm(0:pshead%l_size))
 allocate(pawarray%coreden4pr2(pshead%max_meshsz))
 allocate(pawarray%tcoreden4pr2(pshead%max_meshsz))
 allocate(pawarray%valeden4pr2(pshead%max_meshsz))
 allocate(pawarray%tvaleden4pr2(pshead%max_meshsz))
 allocate(pawarray%ushatden4pr2(pshead%wav_meshsz))
 allocate(pawarray%rvloc0(pshead%max_meshsz))
 allocate(pawarray%vbare(pshead%wav_meshsz))
 allocate(pawarray%kij(pshead%lmn2_size))
 allocate(pawrad%rad(pawrad%max_meshsz))
 allocate(pawrad%radfact(pawrad%max_meshsz))
 allocate(usdata%eig(pshead%basis_size),usdata%keyee(pshead%basis_size))
 allocate(usdata%lval(usdata%nval),usdata%occ(usdata%nval))
 allocate(usdata%rvae(pshead%max_meshsz))
 allocate(usdata%rvloc(pshead%max_meshsz),usdata%rvloc0(pshead%max_meshsz))
 allocate(usdata%rsvale(pshead%max_meshsz))
 allocate(usdata%aesnl(pshead%max_meshsz,usdata%nval))
 allocate(usdata%pawsnl(pshead%max_meshsz,usdata%nval))
 allocate(usdata%snlprj(pshead%basis_size,usdata%nval))
 allocate(usdata%dion(pshead%basis_size,pshead%basis_size))
 allocate(usdata%qijps(pshead%wav_meshsz,pshead%basis_size,pshead%basis_size))

!---- Initialize radial mesh
 call initmesh(pawrad,pshead%log_step,pshead%rad_step)

!---- Read (other) data from USPP
 call rdusps2(pawarray,pawps,pshead,un_log,usdata)
 call dotest(pawarray,pawps,pawrad,pshead,un_log,usdata)

!------------------------------------------------------------------
!---- Compute new data needed by Abinit
!------------------------------------------------------------------
 call calc_valden(pawarray,pawrad,pawps,pshead,usdata,un_log)
 call calc_shapef(pawopt%ishapetyp,pawarray,pawrad,pshead,pawopt%rcompfact,un_log)
 call calc_descr(pawopt%idescr,pawarray,pawrad,pshead,un_log,usdata,pawopt%usexcnhat)
 call calc_scale(pawopt%iscalephi,pawarray,pawps,pawrad,pshead,usdata)
 call calc_core(pawarray,pawrad,pawps,pshead,un_log)
 call calc_vhtnzc(pawarray,pawrad,pawps,pshead,pawopt%ivbare,un_log)
 call calc_dij0(pawarray,pawrad,pawps,pshead,usdata)
 call calc_rhoij0(pawarray,pawps,pshead,usdata)
 call opt_proj(pawps,pawrad,pawrso,pshead,un_log)

!------------------------------------------------------------------
!---- Write PAW pseudopotential file for Abinit
!------------------------------------------------------------------
 call wrpawps(abflname,abunit,pawarray,pawps,pawrad,pshead,pawopt%icompat46,pawopt%ishapetyp)

!------------------------------------------------------------------
!---- Deallocate memory
!------------------------------------------------------------------
 deallocate(pshead%orbitals)
 deallocate(pawps%phi,pawps%tphi,pawps%tproj)
 deallocate(pawps%coredens,pawps%tcoredens,pawps%tvaledens)
 deallocate(pawps%vhtnzc,pawps%rhoij0,pawps%dij0)
 deallocate(pawarray%factscale,pawarray%shapefunc,pawarray%shpnrm)
 deallocate(pawarray%coreden4pr2,pawarray%tcoreden4pr2)
 deallocate(pawarray%valeden4pr2,pawarray%tvaleden4pr2)
 deallocate(pawarray%ushatden4pr2)
 deallocate(pawarray%rvloc0,pawarray%kij)
 deallocate(pawarray%vbare,pawarray%indlmn)
 deallocate(pawrad%rad,pawrad%radfact)
 deallocate(usdata%eig,usdata%keyee,usdata%lval,usdata%snlprj,usdata%dion)
 deallocate(usdata%occ,usdata%aesnl,usdata%pawsnl)
 deallocate(usdata%rsvale,usdata%rvae,usdata%rvloc,usdata%rvloc0,usdata%qijps)

 write(un_log,'(/,a)') '> PAW atomic data file successfully created.'
 write(un_log,'(/,a,/)') '> USPP2Abinit converter ended.'

 end subroutine
!!***
