!{\src2tex{textfont=tt}}
!!****f* ABINIT/wrpawps
!! NAME
!! wrpawps
!!
!! FUNCTION
!! Write a PAW pseudopotential file formatted for Abinit
!!
!! COPYRIGHT
!! Copyright (C) 1998-2007 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  filename= output file name for Abinit
!!  funit= output unit number
!!  ishapetype= option governing the type of shape function
!!  pawarray
!!    %shapefunc(wav_meshsz)= Normalized shape function
!!    %shpnrm(l_size)= Normalisation of shape function for each l
!!  pawrad=datastructure containing mesh size definitions
!!  pawps
!!    %coredens(core_meshsz)= Core density of the atom
!!    %tcoredens(core_meshsz)= Pseudo-core density of the atom
!!    %tvaledens(core_meshsz)= Pseudo-valence density of the atom
!!    %dij0(lmn2_size)= Part of the Dij term calculated in the psp part
!!    %phi(wav_meshsz,basis_size)= atomic partial waves
!!    %tphi(wav_meshsz,basis_size)= atomic pseudo partial waves
!!    %tproj(wav_meshsz,basis_size)= projectors on partial waves
!!    %rhoij0= Atomic initialization of rhoij
!!    %vhtnzc(max_meshsz)= Hartree potential of the ps-density
!!                         of the nucleus + core electrons
!!  pshead
!!    %atomic_charge= Total atomic charge
!!    %basis_size= Number of elements for the paw nl basis
!!    %core_meshsz= Dimension of radial mesh for core and tcore densities
!!    %creatorid= ID of psp generator (here creatorID=1 !)
!!    %l_size= Max. value of l+1 leading to a non zero Gaunt coeffs
!!    %lmax= Maximum value of l
!!    %lmn_size= Number of elements for the paw basis
!!    %log_step= Logarithmic step corresponding to radial mesh
!!    %mesh_type=  Flag defining ther radial grid type
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %prj_meshsz= Dimension of radial mesh for tProj
!!    %pspcod= Psp code number for Abinit (here PAW->pspcod=7 !)
!!    %pspxc_abinit= Abinit s code number for the exchange-correlation
!!    %rad_step= Radial step corresponding to radial mesh
!!    %rc_sph= Default PAW sphere radius
!!    %shape_type= Shape function type (necessarily 2 here)
!!    %sph_meshsz= Dimension of radial mesh for spheres
!!    %title= Title for pseudopotential
!!    %valence_charge= Valence charge
!!    %vale_meshsz= Dimension of radial mesh for vale and tvale densities
!!    %vloc_meshsz= Dimension of radial mesh for vloc=vhtnzc
!!    %vlocopt= option for the formulation of Vloc
!!    %wav_meshsz=Mesh size for partial waves and projectors inherited from uspp
!!  icompat46=Option governing the compatibility with version 4.6.x
!!  ishapetyp=Option governing the type of the shape function k(r)
!!
!! NOTES
!!  File format of formatted PAW psp input for Abinit:
!!  --------------------------------------------------
!!  (1) title (character) line
!!  (2) znucl, zion, pspdat
!!  (3) pspcod, pspxc, lmax, lloc, mmax, r2well
!!  (4) psp_version, creatorID (here, creatorID=2xxx)
!!  (5) basis_size, lmn_size
!!  (6) orbitals (for l=1 to basis_size)
!!  (7) number_of_meshes
!!  For imsh=1 to number_of_meshes
!!      (8)  mesh_index, mesh_type ,sph_meshsz, rad_step[, log_step]
!!  (9) r_cut(SPH)
!!  (10) shape_type, r_shape
!!  For iln=1 to basis_size
!!      (11)  comment(character)
!!      (12) phi(r) (for ir=1 to sph_meshsz(SPH))
!!  For iln=1 to basis_size
!!      (13) comment(character)
!!      (14) tphi(r) (for ir=1 to sph_meshsz(SPH))
!!  For iln=1 to basis_size
!!      (15) comment(character)
!!      (16) tproj(r) (for ir=1 to proj_meshsz(PROJ))
!!  (17) comment(character)
!!  (18) core_density (for ir=1 to core_meshsz(SPH))
!!  (19) comment(character)
!!  (20) tcore_density (for ir=1 to core_meshsz(SPH))
!!  (21) comment(character)
!!  (22) Dij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!!  (23) comment(character)
!!  (24) Rhoij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!!  (25) comment(character)
!!  (26) VHntzc(r) (=Vloc) (for ir=1 to vloc_meshsz(VLOC))
!!  ===== Following lines only if shape_type=-1 =====
!!  For il=1 to 2*max(orbitals)+1
!!      (27) comment(character)
!!      (28) radial mesh index for shapefunc
!!      (29) shapefunc(r)/shpnrm(l)*r**l (for ir=1 to phi_meshsz)
!!  --------------------------------------------------
!!
!! PARENTS
!!      uspp2abinit
!!
!! CHILDREN
!!      date_and_time
!!
!! SOURCE

 subroutine wrpawps(fname,funit,pawarray,pawps,pawrad,pshead,icompat46,ishapetyp)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ---------------------------------------------
 integer :: funit,icompat46,ishapetyp
 character*(fnlen) :: fname
 type(pawarray_type) :: pawarray
 type(pawrad_type)   :: pawrad
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead

!Local variables ---------------------------------------
 integer :: ib,icormsh,iprjmsh,ivalmsh,ivlcmsh,ii,il,ilmn,ir,ival,jlmn,nmesh,outmeshsz
 real(dp) :: dum1,dum2,qr,jbes1,jbes2,al(2),ql(2)
 character*4 :: pspfmt
 character*8 :: strdate
 integer :: meshsz(5)
 real(dp),allocatable :: ff(:)

! *************************************************************************

 pspfmt="paw5"
 if (pshead%vale_meshsz==0) pspfmt="paw3"
 if (pshead%vlocopt/=2)     pspfmt="paw4"

!Size of mesh used for wave functions output
 if (icompat46==0) then
  outmeshsz=min(pshead%sph_meshsz+5,pshead%wav_meshsz)
 else
  outmeshsz=min(pshead%sph_meshsz,pshead%wav_meshsz)
 end if
 nmesh=1;meshsz(1)=outmeshsz

!If tproj use a different mesh, adjust flag iprjmsh
 if (pshead%prj_meshsz==outmeshsz) then
  iprjmsh=1
 else
  nmesh=nmesh+1;iprjmsh=nmesh
  meshsz(nmesh)=pshead%prj_meshsz
 end if

!If (t)coredens use a different mesh, adjust flag icormsh
 if (pshead%core_meshsz<=outmeshsz) then
  icormsh=1
 elseif (pshead%core_meshsz==pshead%prj_meshsz) then
  icormsh=iprjmsh
 else
  nmesh=nmesh+1;icormsh=nmesh
  meshsz(nmesh)=pshead%core_meshsz
 endif

!If Vloc use a different mesh, adjust flag ivlcmsh
 if (pshead%vloc_meshsz==outmeshsz) then
  ivlcmsh=1
 elseif (pshead%vloc_meshsz==pshead%prj_meshsz) then
  ivlcmsh=iprjmsh
 elseif (pshead%vloc_meshsz==pshead%core_meshsz) then
  ivlcmsh=icormsh
 else
  nmesh=nmesh+1;ivlcmsh=nmesh
  meshsz(nmesh)=pshead%vloc_meshsz
 endif

!If (t)valedens use a different mesh, adjust flag ivalmsh
 if (pshead%vale_meshsz<=outmeshsz) then
  ivalmsh=1
 elseif (pshead%vale_meshsz==pshead%prj_meshsz) then
  ivalmsh=iprjmsh
 elseif (pshead%vale_meshsz==pshead%core_meshsz) then
  ivalmsh=icormsh
 elseif (pshead%vale_meshsz==pshead%vloc_meshsz) then
  ivalmsh=ivlcmsh
 else
  nmesh=nmesh+1;ivalmsh=nmesh
  meshsz(nmesh)=pshead%vale_meshsz
 endif

!Open the file for writing
 open(unit=funit,file=trim(fname),form='formatted',status='unknown')

!Write the header
 call date_and_time(strdate)
 write(funit,'(a)') trim(pshead%title)
 write(funit,'(1x,f7.3,1x,f7.3,1x,a,14x,a)') &
&      pshead%atomic_charge,pshead%valence_charge,&
&      trim(strdate),&
&      " : zatom,zion,pspdat"
 write (funit,'(3(1x,i2)," 0 ",i5," 0.",19x,a)') &
&      pshead%pspcod,&
&      pshead%pspxc_abinit,&
&      pshead%lmax,&
&      outmeshsz,&
&      " : pspcod,pspxc,lmax,lloc,mmax,r2well"
 write (funit,'(1x,a4,1x,i4,29x,a)') &
&       pspfmt,pshead%creatorid,&
&       " : pspfmt,creatorID"
 write (funit,'(2(1x,i2),33x,a)') &
&       pshead%basis_size,&
&       pshead%lmn_size,&
&       " : basis_size,lmn_size"
 do ib=1,pshead%basis_size
  write (funit,'(1x,i1)',ADVANCE='NO') pshead%orbitals(ib)
 enddo
 if (pshead%basis_size<20) then
  do ib=pshead%basis_size+1,20
   write (funit,'(a)',ADVANCE='NO') '  '
  enddo
 endif
 write (funit,'(a)') ": orbitals"
 write (funit,'(1x,i1,37x,a)') &
&       nmesh," : number_of_meshes"
 do ii=1,nmesh
  write (funit,'(1x,i1,1x,i1,1x,i4,1x,es22.16,1x,es22.16,a,i1,a)') &
&        ii,2,meshsz(ii),pshead%rad_step,pshead%log_step,&
&           " : mesh ",ii,", type,size,rad_step[,log_step]"
 enddo
 write (funit,'(1x,f13.10,25x,a)') &
&       pshead%rc_sph,&
&       " : r_cut(PAW)"
 if (abs(pshead%rcomp-pshead%rc_sph)<1.d-10) then
  write (funit,'(i2,a,34x,a)') &
&        pshead%shape_type," 0.",&
&        " : shape_type,rshape"
 else
  write (funit,'(i2,1x,f13.10,23x,a)') &
&        pshead%shape_type,pshead%rcomp,&
&        " : shape_type(SIN),rshape"
 end if

!Write partial waves and pseudo partial waves
 do ib=1,pshead%basis_size
  write(funit,'(a,i1,a)') "===== PHI ",ib,&
&      " =====   [phi(r)=PHI(r)/r*Ylm(th,ph)]"
  write(funit,'(a)') " 1  : radial mesh index"
  write(funit,'(3(1x,es23.16))') &
&      (pawps%phi(ir,ib),ir=1,meshsz(1))
 enddo
 do ib=1,pshead%basis_size
  write(funit,'(a,i1,a)') "===== TPHI ",ib,&
&      " =====   [tphi(r)=TPHI(r)/r*Ylm(th,ph)]"
  write(funit,'(a)') " 1  : radial mesh index"
  write(funit,'(3(1x,es23.16))') &
&      (pawps%tphi(ir,ib),ir=1,meshsz(1))
 enddo

!Write projectors
 do ib=1,pshead%basis_size
  write(funit,'(a,i1,a)') "===== TPROJECTOR ",ib,&
&      " =====   [tp(r)=TPROJECTOR(r)/r*Ylm(th,ph)]"
  write(funit,'(1x,i1,a)') iprjmsh,"  : radial mesh index"
  write(funit,'(3(1x,es23.16))') &
&      (pawps%tproj(ir,ib),ir=1,meshsz(iprjmsh))
 enddo

!Write the core density and the pseudo core density
 write(funit,'(a)') "===== CORE_DENSITY ====="
 write(funit,'(1x,i1,a)') icormsh,"  : radial mesh index"
 write(funit,'(3(1x,es23.16))') &
&     (pawps%coredens(ir),ir=1,meshsz(icormsh))
 write(funit,'(a)') "===== PSEUDO_CORE_DENSITY ====="
 write(funit,'(1x,i1,a)') icormsh,"  : radial mesh index"
 write(funit,'(3(1x,es23.16))') &
&     (pawps%tcoredens(ir),ir=1,meshsz(icormsh))

!Write Dij0 and Rhoij0
 write(funit,'(a)') "===== Dij0 ====="
 ii=0
 do jlmn=1,pshead%lmn_size
  write(funit,'(100(1x,es23.16))') &
&     (pawps%dij0(ii+ilmn),ilmn=1,jlmn)
  ii=ii+jlmn
 enddo
 write(funit,'(a)') "===== Rhoij0 ====="
 ii=0
 do jlmn=1,pshead%lmn_size
  write(funit,'(100(1x,es23.16))') &
&     (pawps%rhoij0(ii+ilmn),ilmn=1,jlmn)
  ii=ii+jlmn
 enddo

!Write VHntZC=Vloc
 if (pshead%vlocopt==1) write(funit,'(a)') "===== VHntZC (Vloc(r)) with compensation charge in XC ====="
 if (pshead%vlocopt==2) write(funit,'(a)') "===== VHntZC (Vloc(r)) without compensation charge in XC ====="
 write(funit,'(i2,1x,i2,a)') ivlcmsh,pshead%vlocopt, &
&   "  : radial mesh index, Vloc format (0=Vbare, 1=VH(tnzc) with hat in XC, 2=VH(tnzc) w/o hat in XC)"
 write(funit,'(3(1x,es23.16))') &
&     (pawps%vhtnzc(ir),ir=1,meshsz(ivlcmsh))

!Write (eventually) shape functions
 if (pshead%shape_type==-1) then
  allocate(ff(outmeshsz))
  do il=1,pshead%l_size
   write(funit,'(a,i1,a)') "===== SHAPEF (l=",il-1,") ====="
   write(funit,'(a)') " 1  : radial mesh index"
   if (il==1) then
    ff(1:outmeshsz)=pawarray%shapefunc(1:outmeshsz)
   else
    if (ishapetyp==0.or.ishapetyp==1) then
     do ir=1,outmeshsz
      ff(ir)=pawarray%shapefunc(ir)*pawarray%shpnrm(il)*pawrad%rad(ir)**(il-1)
     enddo
    else if (ishapetyp==2) then
     call shapebes(al,ql,il-1,pshead%rcomp)
     do ir=1,outmeshsz
      if (pawrad%rad(ir)<=pshead%rcomp) then
       qr=ql(1)*pawrad%rad(ir);CALL jbessel(jbes1,dum1,dum2,il-1,0,qr)
       qr=ql(2)*pawrad%rad(ir);CALL jbessel(jbes2,dum1,dum2,il-1,0,qr)
       ff(ir)=al(1)*jbes1+al(2)*jbes2
      else
       ff(ir)=zero
      endif
     enddo
    else
     stop "BUG in wrpawps !"
    endif
   endif
   write(funit,'(3(1x,es23.16))') (ff(ir),ir=1,meshsz(1))
  enddo
  deallocate(ff)
 endif

!Write pseudo valence density
 if (pshead%vale_meshsz>0) then
  write(funit,'(a)') "===== PSEUDO_VALENCE_DENSITY ====="
  write(funit,'(1x,i1,a)') ivalmsh,"  : radial mesh index"
  write(funit,'(3(1x,es23.16))') &
&      (pawps%tvaledens(ir),ir=1,meshsz(ivalmsh))
 end if

!Close the file
 close(funit)

 end subroutine
!!***

